package ac.ed.lurg;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import ac.ed.lurg.country.AbstractCountryAgent;
import ac.ed.lurg.country.GlobalPrice;
import ac.ed.lurg.landuse.CropUsageData;
import ac.ed.lurg.types.CropToDoubleMap;
import ac.ed.lurg.types.CropType;
import ac.ed.lurg.utils.LogWriter;

public class InternationalMarket {

	private Map<CropType, GlobalPrice> worldPrices = new HashMap<CropType, GlobalPrice>();

	public InternationalMarket() {
		//Map<CropType, Double> stockLevel = getInitialStockLevels();

		for (CropType crop : CropType.getImportedTypes()) {
			//Double stock = stockLevel.get(crop);
			worldPrices.put(crop, GlobalPrice.createInitial(crop.getInitialPrice()));
		}
	}
	
	public  Map<CropType, GlobalPrice> getWorldPrices() {
		return worldPrices;
	}

/*	private Map<CropType, Double> getInitialStockLevels() {
		Map<CropType, Double> initialStockLevels = new StockReader().read();
		initialStockLevels.put(CropType.ENERGY_CROPS, 0.0);
		return initialStockLevels;
	} */

	void determineInternationalTrade(Collection<AbstractCountryAgent> countryAgents, double gen2EcDDemand, Timestep timestep) {
		CropToDoubleMap totalImportCommodities = new CropToDoubleMap();
		CropToDoubleMap totalExportCommodities = new CropToDoubleMap();
		CropToDoubleMap totalProduction = new CropToDoubleMap();
		for (AbstractCountryAgent ca : countryAgents) {

			// Get values for world input costs
			Map<CropType, CropUsageData> cropUsage = ca.getCropUsageData();
			
			for (Entry<CropType, CropUsageData> entry : cropUsage.entrySet()) {
				CropType c = entry.getKey();

				double countryNetImports = entry.getValue().getShockedNetImports();
				totalProduction.incrementValue(c, (entry.getValue().getProductionExpected()-entry.getValue().getProductionShock()));
				
				if (countryNetImports > 0)
					totalImportCommodities.incrementValue(c, countryNetImports);
				else
					totalExportCommodities.incrementValue(c, -countryNetImports);
			}
		}

		// energycrops are a special case where demand in global and exogenously specified
		totalImportCommodities.incrementValue(CropType.ENERGY_CROPS, gen2EcDDemand);

		// Look at trade balance and adjust appropriately
		for (CropType crop : CropType.getImportedTypes()) {
		
			GlobalPrice prevPrice = worldPrices.get(crop);
			double imports = totalImportCommodities.containsKey(crop) ? totalImportCommodities.get(crop) : 0.0;
			double exportsBeforeTransportLosses = totalExportCommodities.containsKey(crop) ? totalExportCommodities.get(crop) : 0.0;
			LogWriter.print("Updating " + crop.getGamsName() + " prices : ");
			GlobalPrice adjustedPrice = prevPrice.createWithUpdatedMarketPrices(imports, exportsBeforeTransportLosses, timestep, totalProduction.get(crop));
			LogWriter.println( String.format("Price for %s updated from %s to %s \n", crop.getGamsName(), prevPrice, adjustedPrice));
			worldPrices.put(crop, adjustedPrice);
		}
	}
	
	void writeGlobalMarketFile(Timestep timestep, BufferedWriter outputFile) throws IOException {
		for (CropType crop : CropType.getImportedTypes()) {
			StringBuffer sbData = new StringBuffer();
			GlobalPrice priceQuantity = worldPrices.get(crop);
			sbData.append(String.format("%d,%s", timestep.getYear(), crop.getGamsName()));
			sbData.append(String.format(",%.1f,%.1f", priceQuantity.getImportAmount(), priceQuantity.getExportsAfterTransportLosses()));
			sbData.append(String.format(",%.8f,%.3f", priceQuantity.getExportPrice(), priceQuantity.getStockLevel()));

			outputFile.write(sbData.toString());
			outputFile.newLine();
		}
	}

/*	public double findMaxPriceDiff(Map<CropType, Double> previousExportPrices) {
		if (previousExportPrices == null)
			return Double.MAX_VALUE;
		
		double maxSoFar = 0;
		CropType crop = null;
		
		for (Entry<CropType, GlobalPrice> entry : worldPrices.entrySet()) {
			Double previousP = previousExportPrices.get(entry.getKey());
			
			if (previousP == null)
				return Double.MAX_VALUE;
			
			double diffThisTime = Math.abs(previousP - entry.getValue().getExportPrice());
			if (diffThisTime > maxSoFar) {
				maxSoFar = diffThisTime;
				crop = entry.getKey();
			}
		}
		
		LogWriter.println("findMaxPriceDiff: found " + maxSoFar + " for " + crop);
		return maxSoFar;
	} */
}
