package ac.ed.lurg.demand;

import java.util.HashMap;
import java.util.Map;

import ac.ed.lurg.ModelConfig;
import ac.ed.lurg.country.AgriculturalGDPManager;
import ac.ed.lurg.country.CompositeCountry;
import ac.ed.lurg.country.CompositeCountryManager;
import ac.ed.lurg.country.CountryManager;
import ac.ed.lurg.country.SingleCountry;
import ac.ed.lurg.types.CommodityType;
import ac.ed.lurg.utils.LogWriter;

public abstract class AbstractSSPDemandManager extends AbstractDemandManager {

	protected SspManager sspManager;
	protected String ssp_scenario;
	protected BaseConsumpManager baseConsumpManager;
	protected CalorieManager calorieManager;
	private SingleCountry usa;
	private AgriculturalGDPManager agriculturalGDPManager;

	public AbstractSSPDemandManager(String ssp_scenario, BaseConsumpManager baseConsumpManager,CalorieManager calorieManager, CompositeCountryManager compositeCountryManager) {
		super(compositeCountryManager, calorieManager);
		this.ssp_scenario = ssp_scenario;
		this.baseConsumpManager = baseConsumpManager;
		this.calorieManager = calorieManager;
		sspManager = new SspManager();
		usa = CountryManager.getForName("United States of America");
		agriculturalGDPManager = new AgriculturalGDPManager();
	}
	
	@Override
	public void updateGdpLossesFromShock(CompositeCountry cc, int year, double shockMagnitude) {

		for (SingleCountry c : compositeCountryManager.getAllForCompositeCountry(cc)) {
			SspData sd = sspManager.get(ssp_scenario, year, c);
			LogWriter.println("sd data " + sd);
			if(sd != null) {
				double gdp = sd.getGdp();
				double agriculturalGDPProportion = agriculturalGDPManager.getProportion(c);
				double gdpLost = gdp * agriculturalGDPProportion * shockMagnitude;
				sd.updateShockedGdpLosses(gdpLost);
			}
		}

	}

	@Override
	protected Map<CommodityType, Double> getFoodDemand(SingleCountry c, int year, Map<CommodityType, Double> prices) {
		SspData sd = sspManager.get(ssp_scenario, year, c);
		Map<CommodityType, Double> foodDemandMap = new HashMap<CommodityType, Double>();

		if (sd == null) {
			LogWriter.printlnWarning(String.format("No ssp for %s, baseYr:%d, year:%d, %s, sd:%s. Skipping", ssp_scenario, ModelConfig.BASE_YEAR, year, c.getCountryName(), sd));
			return foodDemandMap;
		}
		
		double gdpPcYear;
		if((ModelConfig.CONSTANT_DIET_HIGH_INCOME & c.getIncomeGroup().toLowerCase().contains("high")) || 
				(ModelConfig.CONSTANT_DIET_LOW_INCOME & !c.getIncomeGroup().toLowerCase().contains("high")))
			gdpPcYear = sspManager.get(ssp_scenario, ModelConfig.BASE_YEAR, c).getShockedGdpPc();

		else gdpPcYear= sd.getShockedGdpPc();
		
		SspData usaSd = sspManager.get(ssp_scenario, year, usa);
		
		LogWriter.println("Got ssp data for " + c.getCountryName() + " of " + sd);
		foodDemandMap = getFoodDemandMap(c, year, gdpPcYear * ModelConfig.SSP_GDP_PC_FACTOR, sd.getPopulation() * ModelConfig.SSP_POPULATION_FACTOR, prices, usaSd.getGdpPc());
		
		if(!ModelConfig.SHOCKS_POSSIBLE){
			return foodDemandMap;
		}
		return dietaryAdjustment(foodDemandMap, year, calorieManager.get(c));
	}
	
	protected double rebaseConsumption(double baseGdpPc, double baseCpc, double baseExpCpc, double gdpPc, double newExpCpc) {
		double gapRatio = Math.exp((gdpPc-baseGdpPc)/baseGdpPc * ModelConfig.DIETARY_CLOSURE_PARAM);
		if (gapRatio>1) gapRatio = 1;
		
		double adjFactor = (baseExpCpc==0 || ModelConfig.DONT_REBASE_DEMAND) ? 0.0 : (baseCpc - baseExpCpc)*gapRatio;  // if baseExpCpc is zero no adjustment
		double projectedCpc = newExpCpc + adjFactor;
		
		return projectedCpc;
	}

	abstract Map<CommodityType, Double> getFoodDemandMap(SingleCountry c, int year, double gdpPc, double population, Map<CommodityType, Double> prices, double usaGdpPc);
}
